#version 300 es
precision mediump float; // 添加默认精度

const mediump vec3 satLuminanceWeighting = vec3(0.2126, 0.7152, 0.0722);
const mediump vec3 hsLuminanceWeighting = vec3(0.3, 0.3, 0.3);
const mediump vec3 warmFilter = vec3(0.93, 0.54, 0.0);

const mediump mat3 RGBtoYIQ = mat3(
    0.299, 0.587, 0.114,
    0.596, -0.274, -0.322,
    0.212, -0.523, 0.311
);

const mediump mat3 YIQtoRGB = mat3(
    1.0, 0.956, 0.621,
    1.0, -0.272, -0.647,
    1.0, -1.105, 1.702
);

const mediump vec4 kRGBToYPrime = vec4(0.299, 0.587, 0.114, 0.0);
const mediump vec4 kRGBToI = vec4(0.595716, -0.274453, -0.321263, 0.0);
const mediump vec4 kRGBToQ = vec4(0.211456, -0.522591, 0.31135, 0.0);
const mediump vec4 kYIQToR = vec4(1.0, 0.9563, 0.6210, 0.0);
const mediump vec4 kYIQToG = vec4(1.0, -0.2721, -0.6474, 0.0);
const mediump vec4 kYIQToB = vec4(1.0, -1.1070, 1.7046, 0.0);

in vec2 fTexCoord;
out vec4 fragColor;
uniform sampler2D uTexture;

uniform int width;
uniform int height;
uniform float brightness;   // 亮度调整参数，范围可以是从-1.0（更暗）到1.0（更亮）
uniform float contrast;     // 对比度调整参数，大于1.0增加对比度，小于1.0减少对比度，等于1.0时无变化
uniform float saturation;   // 饱和度调整参数，大于1.0增加饱和度，小于1.0减少饱和度，等于1.0时无变化
uniform float shadows;      // 调节阴影， 默认为1
uniform float highlights;   // 调节高光，默认为1
uniform float temperature;  // 色温调整, 默认为0
uniform float hue;          // 色调调整，默认为0
uniform float sharpen;      // 锐化，默认为0
uniform float beautyDerma;  // 美颜磨皮，默认为0
uniform float beautyWhite;  // 美颜美白，默认为0

void colorHueWarth(inout vec4 color, float hueValue, float warthValue) {
    // 转换为YIQ
    highp float YPrime = dot(color, kRGBToYPrime);
    highp float I = dot(color, kRGBToI);
    highp float Q = dot(color, kRGBToQ);
    // 计算色相和色度
    highp float hueAngle = atan(Q, I);
    highp float chroma = sqrt(I * I + Q * Q);
    // 调整色相
    hueAngle += hueValue;
    // 转换回YIQ
    Q = chroma * sin(hueAngle);
    I = chroma * cos(hueAngle);
    // 转换回RGB
    highp vec4 yIQ = vec4(YPrime, I, Q, 0.0);
    color.r = dot(yIQ, kYIQToR);
    color.g = dot(yIQ, kYIQToG);
    color.b = dot(yIQ, kYIQToB);
    // 调整色温
    mediump vec3 yiq = RGBtoYIQ * color.rgb;
    lowp vec3 rgb = YIQtoRGB * yiq;
    lowp vec3 processed = vec3(
        (rgb.r < 0.5 ? (2.0 * rgb.r * warmFilter.r) : (1.0 - 2.0 * (1.0 - rgb.r) * (1.0 - warmFilter.r))),
        (rgb.g < 0.5 ? (2.0 * rgb.g * warmFilter.g) : (1.0 - 2.0 * (1.0 - rgb.g) * (1.0 - warmFilter.g))),
        (rgb.b < 0.5 ? (2.0 * rgb.b * warmFilter.b) : (1.0 - 2.0 * (1.0 - rgb.b) * (1.0 - warmFilter.b)))
    );
    color.rgb = mix(rgb, processed, warthValue);
}

vec4 colorSharpen(vec2 uv, vec4 color, float sharpenValue) {
    float kernel[9];
    vec2 tex_offset = 1.0 / vec2(textureSize(uTexture, 0)); // 获取纹理大小的倒数
    // 使用锐化强度参数调整锐化内核
    kernel[0] = 0.0; kernel[1] = -1.0 * sharpenValue; kernel[2] = 0.0;
    kernel[3] = -1.0 * sharpenValue; kernel[4] = 4.0 * sharpenValue + 1.0; kernel[5] = -1.0 * sharpenValue;
    kernel[6] = 0.0; kernel[7] = -1.0 * sharpenValue; kernel[8] = 0.0;

    vec3 newColor = vec3(0.0);
    for (int i = 0; i < 3; i++) {
        for (int j = 0; j < 3; j++) {
            vec3 sampleTex = texture(uTexture, fTexCoord + tex_offset * vec2(i - 1, j - 1)).rgb;
            newColor += sampleTex * kernel[i * 3 + j];
        }
    }
    return vec4(newColor, color.a);
}

void beauty(inout vec4 color, float dermaValue, float whiteValue) {
    if (dermaValue < 0.01) {
        color.rgb = color.rgb + whiteValue;
    } else {
        float x_a = float(width);
        float y_a = float(height);

        float mul_x = 2.0 / x_a;
        float mul_y = 2.0 / y_a;
        vec2 blurCoordinates0 = fTexCoord + vec2(0.0 * mul_x, -10.0 * mul_y);
        vec2 blurCoordinates2 = fTexCoord + vec2(8.0 * mul_x, -5.0 * mul_y);
        vec2 blurCoordinates4 = fTexCoord + vec2(8.0 * mul_x, 5.0 * mul_y);
        vec2 blurCoordinates6 = fTexCoord + vec2(0.0 * mul_x, 10.0 * mul_y);
        vec2 blurCoordinates8 = fTexCoord + vec2(-8.0 * mul_x, 5.0 * mul_y);
        vec2 blurCoordinates10 = fTexCoord + vec2(-8.0 * mul_x, -5.0 * mul_y);

        mul_x = 1.8 / x_a;
        mul_y = 1.8 / y_a;
        vec2 blurCoordinates1 = fTexCoord + vec2(5.0 * mul_x, -8.0 * mul_y);
        vec2 blurCoordinates3 = fTexCoord + vec2(10.0 * mul_x, 0.0 * mul_y);
        vec2 blurCoordinates5 = fTexCoord + vec2(5.0 * mul_x, 8.0 * mul_y);
        vec2 blurCoordinates7 = fTexCoord + vec2(-5.0 * mul_x, 8.0 * mul_y);
        vec2 blurCoordinates9 = fTexCoord + vec2(-10.0 * mul_x, 0.0 * mul_y);
        vec2 blurCoordinates11 = fTexCoord + vec2(-5.0 * mul_x, -8.0 * mul_y);

        mul_x = 1.6 / x_a;
        mul_y = 1.6 / y_a;
        vec2 blurCoordinates12 = fTexCoord + vec2(0.0 * mul_x, -6.0 * mul_y);
        vec2 blurCoordinates14 = fTexCoord + vec2(-6.0 * mul_x, 0.0 * mul_y);
        vec2 blurCoordinates16 = fTexCoord + vec2(0.0 * mul_x, 6.0 * mul_y);
        vec2 blurCoordinates18 = fTexCoord + vec2(6.0 * mul_x, 0.0 * mul_y);

        mul_x = 1.4 / x_a;
        mul_y = 1.4 / y_a;
        vec2 blurCoordinates13 = fTexCoord + vec2(-4.0 * mul_x, -4.0 * mul_y);
        vec2 blurCoordinates15 = fTexCoord + vec2(-4.0 * mul_x, 4.0 * mul_y);
        vec2 blurCoordinates17 = fTexCoord + vec2(4.0 * mul_x, 4.0 * mul_y);
        vec2 blurCoordinates19 = fTexCoord + vec2(4.0 * mul_x, -4.0 * mul_y);

        float central;
        float gaussianWeightTotal;
        float sum;
        float distanceFromCentralColor;
        float gaussianWeight;

        float distanceNormalizationFactor = 3.6;

        central = texture(uTexture, fTexCoord).g;
        gaussianWeightTotal = 0.2;
        sum = central * 0.2;

        float colorG = texture(uTexture, blurCoordinates0).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates1).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates2).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates3).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates4).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates5).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates6).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates7).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates8).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates9).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates10).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates11).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates12).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.1 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates13).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.1 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates14).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.1 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates15).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.1 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates16).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.1 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates17).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.1 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates18).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.1 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        colorG = texture(uTexture, blurCoordinates19).g;
        distanceFromCentralColor = min(abs(central - colorG) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.1 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += colorG * gaussianWeight;

        sum = sum / gaussianWeightTotal;

        colorG = color.g - sum + 0.5;

        // 高反差保留
        for (int i = 0; i < 5; ++i) {
            if (colorG <= 0.5) {
                colorG = colorG * colorG * 2.0;
            } else {
                colorG = 1.0 - ((1.0 - colorG) * (1.0 - colorG) * 2.0);
            }
        }

        float aa = 1.0 + pow(sum, 0.3) * 0.09;
        vec3 smoothColor = (color.rgb + vec3(beautyWhite)) * aa - vec3(colorG) * (aa - 1.0);
        smoothColor = clamp(smoothColor, vec3(0.0), vec3(1.0));

        smoothColor = mix(color.rgb + vec3(beautyWhite), smoothColor, pow(color.g, 0.33));
        smoothColor = mix(color.rgb + vec3(beautyWhite), smoothColor, pow(color.g, 0.39));

        smoothColor = mix(color.rgb + vec3(beautyWhite), smoothColor, beautyDerma);
        color = vec4(pow(smoothColor, vec3(0.96)), color.a);
    }
}

void main() {
    vec4 textureColor = texture(uTexture, fTexCoord);
    // 锐化比较特殊，先进行锐化
    vec4 resultColor = colorSharpen(fTexCoord, textureColor, sharpen);
    float toolEpsilon = 0.005;

    // 调节亮度
    resultColor.rgb = clamp(resultColor.rgb + vec3(brightness, brightness, brightness), 0.0, 1.0);

    // 调节对比度
    resultColor.rgb = ((resultColor.rgb - 0.5) * max(contrast, 0.0)) + 0.5;
    resultColor = vec4(clamp(((resultColor.rgb - vec3(0.5)) * contrast + vec3(0.5)), 0.0, 1.0), resultColor.a);

    // 调节饱和度
    lowp float satLuminance = dot(resultColor.rgb, satLuminanceWeighting);
    lowp vec3 greyScaleColor = vec3(satLuminance);
    resultColor = vec4(clamp(mix(greyScaleColor, resultColor.rgb, saturation), 0.0, 1.0), resultColor.a);

    // 调节高光、阴影
    if (abs(highlights - 1.0) > toolEpsilon || abs(shadows - 1.0) > toolEpsilon) {
        mediump float hsLuminance = dot(resultColor.rgb, hsLuminanceWeighting);
        mediump float shadow = clamp((pow(hsLuminance, 1.0 / shadows) + (-0.76) * pow(hsLuminance, 2.0 / shadows)) - hsLuminance, 0.0, 1.0);
        mediump float highlight = clamp((1.0 - (pow(1.0 - hsLuminance, 1.0 / (2.0 - highlights)) + (-0.8) * pow(1.0 - hsLuminance, 2.0 / (2.0 - highlights)))) - hsLuminance, -1.0, 0.0);
        lowp vec3 hsresult = vec3(0.0, 0.0, 0.0) + ((hsLuminance + shadow + highlight) - 0.0) * ((resultColor.rgb - vec3(0.0, 0.0, 0.0)) / (hsLuminance - 0.0));
        mediump float contrastedLuminance = ((hsLuminance - 0.5) * 1.5) + 0.5;
        mediump float whiteInterp = contrastedLuminance * contrastedLuminance * contrastedLuminance;
        mediump float whiteTarget = clamp(highlights, 1.0, 2.0) - 1.0;
        hsresult = mix(hsresult, vec3(1.0), whiteInterp * whiteTarget);
        mediump float invContrastedLuminance = 1.0 - contrastedLuminance;
        mediump float blackInterp = invContrastedLuminance * invContrastedLuminance * invContrastedLuminance;
        mediump float blackTarget = 1.0 - clamp(shadows, 0.0, 1.0);
        hsresult = mix(hsresult, vec3(0.0), blackInterp * blackTarget);
        resultColor = vec4(hsresult.rgb, resultColor.a);
    }

    // 调节色调、色温
    colorHueWarth(resultColor, hue, temperature);

    // 美颜
    beauty(resultColor, beautyDerma, beautyWhite);

    // 预乘alpha
    resultColor.rgb *= textureColor.a;

    fragColor = resultColor;
}